#
# AtlantisBlueCMS CMSFolder.py
#
# Copyright 2003 Atlantis Blue Pty Ltd. See LICENSE.txt for more info.
#
####

import string
from types import StringType
from DateTime import DateTime
from Globals import ImageFile, DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from OFS.Folder import Folder
from webdav.WriteLockInterface import WriteLockInterface
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.OrderedFolder.OrderedObjectManager import OrderedObjectManager
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport
from Products.OrderedFolder.TransparencySupport import TransparencySupport
from Products.OrderedFolder.MaxItemsSupport import MaxItemsSupport
from Products.OrderedFolder.OrderedFolder import OrderedFolder

security = ClassSecurityInfo()
oomPermission = 'Copy or Move'

factory_type_information = ( { 'id'             : 'CMS Folder'
                             , 'meta_type'      : 'CMS Folder'
                             , 'add list name'  : 'CMS Folder'
                             , 'description'    : """ Default CMS Container """
                             , 'icon'           : 'folder_icon.gif'
                             , 'product'        : 'AtlantisBlueCMS'
                             , 'factory'        : 'addCMSFolder'
                             , 'filter_content_types' : 0
                             , 'immediate_view' : 'folder_listing'
                             , 'actions'        :
                                ( { 'id'            : 'view' 
                                  , 'name'          : 'View'
                                  , 'action'        : ''
                                  , 'permissions'   :
                                     (CMFCorePermissions.View,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'folderlisting'
                                  , 'name'          : 'Folder Listing'
                                  , 'action'        : 'folder_listing'
                                  , 'permissions'   :
                                     (CMFCorePermissions.Permissions.access_contents_information,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'local_roles'
                                  , 'name'          : 'Local Roles'
                                  , 'action'        : 'folder_localrole_form'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'folder_edit_form'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'folder'
                                  }
                                )
                             }
                           ,
                           )


#class CMSFolder(OrderedFolder, PortalFolder):
# note: tried SkinnedFolder, but it sets index_html = None :(
# using CMFCatalogAware and Portal folder instead, as SkinnedFolder does.
# moved the CMFCatalogAware and PortalContent to CMSPage 
#
class CMSFolder(TransparencySupport, 
		OrderedObjectManager, 
		MaxItemsSupport, 
		PortalFolder):
    """
    Atlantis Blue CMS - Base Container Object.
    """

    # note: see http://www.zope.org/Members/digitalis/basic_cmf_product/how_to 

    content_id 			= 'CMS Folder' 
    meta_type			= 'CMS Folder'
    content_description 	= 'A base container for CMS page elements'

    # portal_type should be set per-insance by the CMF factory
    #portal_type			= 'CMS Folder'

    _properties = PortalFolder._properties + TransparencySupport._properties + MaxItemsSupport._properties

    __implements__ = (DefaultDublinCoreImpl.__implements__ ,
                      WriteLockInterface)

    security = ClassSecurityInfo()

    manage_options = PortalFolder.manage_options 
 
    security.declareProtected( CMFCorePermissions.View, 'view' )
    view = SkinnedFolder.__call__

    cms_main_template		= PageTemplateFile('ab_cms/cms_main_template', globals())
    cms_main_css		= DTMLFile('ab_cms/cms_main_css', globals())
    main_css			= DTMLFile('ab_cms/main_css', globals())
    cms_confirm_dialog		= DTMLFile('ab_cms/cms_confirm_dialog', globals())
    cms_site_properties_form	= PageTemplateFile('ab_cms/cms_site_properties_form', globals())
    cms_view_template		= PageTemplateFile('ab_cms/cms_view_template', globals())
    cms_content_view_macros	= PageTemplateFile('ab_cms/cms_content_view_macros', globals())
    cms_widget_template		= PageTemplateFile('ab_cms/cms_widget_template', globals())
    cms_object_edit		= PageTemplateFile('ab_cms/cms_object_edit', globals())
    cms_page_edit		= PageTemplateFile('ab_cms/cms_page_edit', globals())
    cms_page_preview		= PageTemplateFile('ab_cms/cms_page_preview', globals())
    cms_page_properties		= PageTemplateFile('ab_cms/cms_page_properties', globals())
    cms_logo_ab			= ImageFile('ab_cms/cms-logo-ab.png', globals())
    cms_logo_cms		= ImageFile('ab_cms/cms-logo-cms.png', globals())
    cms_logo			= ImageFile('ab_cms/cms-logo.png', globals())
    cms_button_edit_off		= ImageFile('ab_cms/cms_button_edit_off.gif', globals())
    cms_button_edit_on		= ImageFile('ab_cms/cms_button_edit_on.gif', globals())
    cms_button_preview_off	= ImageFile('ab_cms/cms_button_preview_off.gif', globals())
    cms_button_preview_on	= ImageFile('ab_cms/cms_button_preview_on.gif', globals())
    cms_button_properties_off	= ImageFile('ab_cms/cms_button_properties_off.gif', globals())
    cms_button_properties_on	= ImageFile('ab_cms/cms_button_properties_on.gif', globals())
    cms_button_up		= ImageFile('ab_cms/cms_button_up.gif', globals())
    cms_button_up_inactive	= ImageFile('ab_cms/cms_button_up_inactive.gif', globals())
    cms_button_dn		= ImageFile('ab_cms/cms_button_dn.gif', globals())
    cms_button_dn_inactive	= ImageFile('ab_cms/cms_button_dn_inactive.gif', globals())
    cms_button_x		= ImageFile('ab_cms/cms_button_x.gif', globals())
    cms_button_x_inactive	= ImageFile('ab_cms/cms_button_x_inactive.gif', globals())
    cms_button_nl		= ImageFile('ab_cms/cms_button_nl.gif', globals())
    cms_pdf_icon		= ImageFile('ab_cms/cms_pdf.gif', globals())
    cms_published_icon		= ImageFile('ab_cms/cms_published.png', globals())
    cms_unpublished_icon	= ImageFile('ab_cms/cms_unpublished.png', globals())
    cms_uncommitted_dot		= ImageFile('ab_cms/cms_uncommitted_dot.png', globals())
    cms_icon_print_small	= ImageFile('ab_cms/cms_icon_print_small.gif', globals())
    cms_icon_pdf_large		= ImageFile('ab_cms/cms_icon_pdf_large.gif', globals())
    cms_icon_pdf_small		= ImageFile('ab_cms/cms_icon_pdf_small.gif', globals())
    cms_icon_doc_large		= ImageFile('ab_cms/cms_icon_doc_large.gif', globals())
    cms_icon_xls_large		= ImageFile('ab_cms/cms_icon_xls_large.gif', globals())
    cms_icon_img_large		= ImageFile('ab_cms/cms_icon_img_large.gif', globals())
    cms_icon_unk_large		= ImageFile('ab_cms/cms_icon_unk_large.gif', globals())

    # override the ZMI folder view
    manage_main = DTMLFile('dtml/manage_main', globals())

    misc_ = {'up'     : ImageFile('zmi/up.gif', globals()),
             'down'   : ImageFile('zmi/down.gif', globals()),
             'top'    : ImageFile('zmi/top.gif', globals()),
             'bottom' : ImageFile('zmi/bottom.gif', globals())}

    def cmsAddContentFolders(self):
     content_folders  = ['inline_content', 'CMS_files', 'CMS_other' ]
     function_folders = []
     for folder_id in content_folders:
      if not folder_id in self.objectIds('CMS Folder'):
        self.invokeFactory(id=folder_id, type_name='CMS Folder')
        newfolder=getattr(self, folder_id)
        newfolder.manage_changeProperties(_is_transparent='1')
        newfolder.manage_changeProperties(maximumItems='20')
    
     for folder_id in function_folders:
      if not folder_id in container_folder.objectIds('CMS Folder'):
       container_folder.invokeFactory(id=folder_id, type_name='CMS Folder')

    def folderList(self,object,max_depth=10,current_depth=0):
     folders = []
     current_depth=current_depth+1
     for child in object.objectValues(['CMS Page']):
       if not hasattr(child, 'cms_skip_explorer'):
         folders.append(child)
         if current_depth < max_depth:
           calls = child.folderList(child, max_depth=max_depth, current_depth=current_depth)
           for item in calls:
              if item:
                 folders.append(item) 
     return folders

    def linkdepth(self, url):
     siteRoot = self.getSiteRoot()
     siteRootURL = siteRoot.absolute_url()
     siteRootDepth = string.count(siteRootURL,'/')-2
     hereDepth = string.count(url,'/')-2
     return hereDepth - siteRootDepth + 1

    def cms_move_object_down(self, REQUEST):
     """ Move object up """
     id = REQUEST.id
     self.move_objects_down(id)
     return self.cmsadmin()
    def cms_move_object_up(self, REQUEST):
     """ Move object down """
     id = REQUEST.id
     self.move_objects_up(id)
     return self.cmsadmin()
    def cms_delete_object(self, object_id):
     """ Delete object """
     # note: this is a hack because apparently manage_delObject doesn't work with
     #       TransparentFolders. Objects in the inline_content_tmp and CMS_files
     #       folders should be unique in any case because of their transparency
     if getattr(self, 'inline_content_tmp', None):
       if getattr(self.inline_content_tmp, object_id, None):
         self.inline_content_tmp.manage_delObjects([object_id])
     if getattr(self.CMS_files, object_id, None):
       self.CMS_files.manage_delObjects([object_id])
     return self.cmsadmin()

    def cms_get_cms_types(self):
     cms_types = []
     for line in self.CMS_content_types:
      cms_types.append(string.split(line, ',')[0])
     return cms_types
    def cms_get_edit_object_cms_type(self, object_id):
      if object_id:
        edit_object = getattr(self, object_id)
        cms_types = {}
        for line in self.CMS_content_types:
         cms_types[string.split(line, ',')[2]] = string.split(line, ',')[0]
        cms_type = cms_types[edit_object.meta_type]
      else:
        cms_type = None
      return cms_type
    def cms_get_display_type(self, cms_type):
     cms_display_type = {}
     for line in self.CMS_content_types:
      cms_display_type[string.split(line, ',')[0]] = string.split(line, ',')[1]
     return cms_display_type[cms_type]
    def cms_get_meta_type(self, cms_type):
     cms_meta_type = {}
     for line in self.CMS_content_types:
      cms_meta_type[string.split(line, ',')[0]] = string.split(line, ',')[2]
     return cms_meta_type[cms_type]
    def cms_get_view_pt_name(self, object_id):
     cms_view_pt_name = {}
     meta_type = getattr(self,object_id).meta_type
     for line in self.CMS_content_types:
      cms_view_pt_name[string.split(line, ',')[2]] = string.split(line, ',')[3]
     return cms_view_pt_name[meta_type]
    def cms_get_title_display(self, cms_type):
     cms_meta_type = {}
     for line in self.CMS_content_types:
      cms_meta_type[string.split(line, ',')[0]] = string.split(line, ',')[4]
     return cms_meta_type[cms_type]
    def cms_get_file_status(self, cms_type):
     cms_meta_type = {}
     for line in self.CMS_content_types:
      cms_meta_type[string.split(line, ',')[0]] = string.split(line, ',')[5]
     return cms_meta_type[cms_type]

    def cms_get_object_by_id(self, object_id):
     try:
      obj = getattr(self,object_id)
     except:
      obj = None
     return obj
    def cms_get_text_by_id(self, object_id):
     try:
      obj = getattr(self, object_id)
     except:
      obj = None
     try:
      src = obj.text
     except:
      src = None
     return src

    def cms_get_icon_abbr(self, content_type=None):
     try:
	icon_abbr = self.icon_abbrs[content_type]
     except:
	icon_abbr = 'unk'
     return icon_abbr
      
    def cmsGetEditObjectById(self, object_id):
     try:
      obj = getattr(self.inline_content_tmp,object_id)
     except:
      obj = getattr(self.inline_content,object_id)
     return obj

    def cmsGetFrontEndObjectById(self, object_id):
      return getattr(self.inline_content,object_id)

    def cmsSaveChanges(self, REQUEST):
     #Atlantis Blue CMS - Wrapper around all CMF Factory, CMF Metadata edit and Zope Property edit calls
     request = REQUEST
     RESPONSE =  request.RESPONSE
     return_message = ''
     precondition=None

     try:
      file = request.file
     except: 
      file = ''

     try:
      cms_action = request.cms_action
     except: 
      cms_action = None
    
     try:
      cms_type = request.edit_object_cms_type
     except: 
      cms_type = None
    
     try:
      object_id = request.object_id
     except:
      if file:
        object_id=getattr(file,'filename', '')
      else: 
        object_id='%s_%s'% (cms_type, DateTime().strftime('%Y_%m_%d_%H%M%S'))
    
     try:
      object_title = request.title
     except: 
      object_title = None
    
     try:
      object_description = request.description
     except: 
      object_description = None
    
     try:
      object_subject = request.subject
     except: 
      object_subject = None
    
     try:
      text = request.form_text
     except: 
      text = None
    
     if cms_action == 'add':
      if cms_type == 'page':
        self.invokeFactory(id=object_id, type_name='CMS Page')
        obj=getattr(self, object_id)
        obj.edit(title=object_title)
    
      if cms_type == 'text':
        content_folder = self.cmsPageVersionCheck()
        content_folder.invokeFactory(id=object_id, type_name='Document')
        document = getattr(content_folder , object_id )
        document.edit( text_format='structured-text'
                     , text=text
		     , file=file
		     )
        if document.hasProperty('editMetaData'):
           content.editMetadata( **kw )
      if cms_type == 'graphic':
        content_folder = self.cmsPageVersionCheck()
        content_folder.invokeFactory(id=object_id, type_name='Image')
        obj=getattr(content_folder, object_id)
        obj.edit(file=file)
        obj.editMetadata(title=object_title)
      if cms_type == 'file':
        content_folder = self.CMS_files
        content_folder.invokeFactory(id=object_id, type_name='File')
        obj=getattr(content_folder, object_id)
        obj.edit(file=file)
        obj.editMetadata(title=object_title)

     if cms_action == 'edit':
      if cms_type == 'page':
        self.edit(title=object_title, description=object_description)
      if cms_type == 'text':
        content_folder = self.cmsPageVersionCheck()
        document=getattr(content_folder, object_id)
        document.edit( text_format='structured-text'
                     , text=text
		     )
      if cms_type == 'graphic':
        content_folder = self.cmsPageVersionCheck()
	graphic = getattr(content_folder, object_id)
	if file:
	  graphic.edit( precondition=precondition, file=file )
        content_folder.plone_utils.contentEdit( graphic
                                   , id=object_id
                                   , title=object_title
                                   , description=object_description )
        content_folder.plone_utils.editMetadata( graphic
                                   , title=object_title
                                   , description=object_description )
      #return_message = return_message + "<script type='text/javascript' language='JavaScript'> alert('contentEdit and editMetadata executed for graphic.'); </script>"
      return_message = return_message + ''
      if cms_type == 'file':
        content_folder = self.CMS_files
	obj = getattr(content_folder, object_id)
	if file:
	  obj.edit( precondition=precondition, file=file )
        obj.plone_utils.contentEdit( obj
                                   , id=object_id
                                   , title=object_title
                                   , description=object_description )

      #return_message = return_message + "<script type='text/javascript' language='JavaScript'> alert('Changes saved." + cms_action + "; " + cms_type + " '); </script>"
      return_message = return_message + ''

     return return_message
    
    def standard_error_message():
     #  you could easily check for the error_type and
     #  dispatch to an appropriate PageTemplate.
    
     error_type=kwargs.get('error_type', None)
     error_message=kwargs.get('error_message', None)
     error_log_url=kwargs.get('error_log_url', None)
     error_tb=kwargs.get('error_tb', None)
     error_traceback=kwargs.get('error_traceback', None)
     error_value=kwargs.get('error_value', None)
    
     error_page=None
    
     error_page=context.default_error_message(error_type=error_type,
                                              error_message=error_message,
                                              error_tb=error_tb,
                                              error_value=error_value,
					      error_traceback=error_traceback)
     return error_page

# note: this is a hack to allow __init__.py/context.registerClass to work so copy/paste support works
addCMSFolderForm = DTMLFile('dtml/manage_main', globals())
meta_type			= 'CMS Folder'

security.declareProtected(CMFCorePermissions.AddPortalFolders, 'addCMSFolder')
def addCMSFolder( self, id, title='', description='', REQUEST=None ):
    """ adds a CMS Folder """
    sf = CMSFolder( id, title=title)
    sf.description=description
    self._setObject( id, sf )
    if REQUEST is not None:
       REQUEST['RESPONSE'].redirect( sf.absolute_url() + '/manage_main' )

InitializeClass(CMSFolder)
