#
# AtlantisBlueCMS CMSPage.py
#
# Copyright 2003 Atlantis Blue Pty Ltd. See LICENSE.txt for more info.
#
####

import string
from urllib2 import Request
from AccessControl import ClassSecurityInfo
import Acquisition
from Globals import InitializeClass, DTMLFile
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
#from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.AtlantisBlueCMS.PortalContent import PortalContent
from Products.AtlantisBlueCMS.CMSFolder import CMSFolder

security 			= ClassSecurityInfo()
oomPermission = 'Copy or Move'

factory_type_information = ( { 'id'             : 'CMS Page'
                             , 'meta_type'      : 'CMS Page'
                             , 'description'    : """ CMS Page Container """
                             , 'icon'           : 'folder_icon.gif'
                             , 'product'        : 'AtlantisBlueCMS'
                             , 'factory'        : 'addCMSPage'
                             , 'filter_content_types' : 0
                             , 'immediate_view' : 'cmsadmin'
                             , 'actions'        :
                                ( { 'id'            : 'view' 
                                  , 'name'          : 'View'
                                  , 'action'        : 'index_html'
                                  , 'permissions'   :
                                     (CMFCorePermissions.View,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'cmsadmin'
                                  , 'name'          : 'CMS Admin'
                                  , 'action'        : 'cmsadmin'
                                  , 'permissions'   :
                                     (CMFCorePermissions.Permissions.access_contents_information,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'page.pdf'
                                  , 'name'          : 'Generate PDF Page'
                                  , 'action'        : 'page.pdf'
                                  , 'permissions'   :
                                     (CMFCorePermissions.Permissions.access_contents_information,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'book.pdf'
                                  , 'name'          : 'Generate PDF Book'
                                  , 'action'        : 'book.pdf'
                                  , 'permissions'   :
                                     (CMFCorePermissions.Permissions.access_contents_information,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'folderlisting'
                                  , 'name'          : 'Folder Listing'
                                  , 'action'        : 'folder_listing'
                                  , 'permissions'   :
                                     (CMFCorePermissions.Permissions.access_contents_information,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'local_roles'
                                  , 'name'          : 'Local Roles'
                                  , 'action'        : 'folder_localrole_form'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'folder_edit_form'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'folder'
                                  }
                                )
                             }
                           ,
                           )    
    
class CMSPage(CMFCatalogAware, 
              CMSFolder, 
              PortalContent):
    """
    Atlantis Blue CMS - Page Element Container Object.
    """

    meta_type			= 'CMS Page'
    _properties			= CMSFolder._properties + ({'id':'_is_published', 'type': 'boolean', 'mode': 'w'},)

    manage_options = CMSFolder.manage_options + \
                     CMFCatalogAware.manage_options

    isPortalContent = 1
    _isPortalContent = 1  # More reliable than 'isPortalContent'.

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    effective_date = expiration_date = None

    security 			= ClassSecurityInfo()

    def __init__( self, id, title='', description='' ):
        """Initialize CMSPage""" 
        self.id = id
        self.title = title
        self.description = description

    def cmsPageVersionCheck(self):
     inline_content = getattr(Acquisition.aq_self(self), 'inline_content')
     if not 'inline_content_tmp' in self.objectIds():
       #self.manage_clone( ob=inline_content, id='inline_content_tmp')
       # note: the below is an ugly hack because manage_clone is throwing
       #       an error in _verifyObjectPaste mt_permission or validate_src
        inline_content_tmp=inline_content._getCopy(self)
        inline_content_tmp._setId('inline_content_tmp')
        self._setObject('inline_content_tmp', inline_content_tmp)
        inline_content_tmp=inline_content_tmp.__of__(self)
        inline_content_tmp.manage_afterClone(inline_content_tmp)
     content_folder = getattr(self, 'inline_content_tmp')
     return content_folder

    def cmsPageContentChangeCommit(self):
      if 'inline_content_tmp' in self.objectIds():
	self.manage_delObjects('inline_content')
        #self.manage_deleteObject( id='inline_content')
        #inline_content_tmp=self._getOb('inline_content_tmp')
        self.manage_renameObject( 'inline_content_tmp', 'inline_content')
        # note: the below is an ugly hack because manage_renameObject is throwing
        #       an error in _verifyObjectPaste mt_permission or validate_src
        #inline_content_new=self._getOb('inline_content_tmp')
	#self.manage_delObjects('inline_content')
        #inline_content_new = Acquisition.aq_base(inline_content_new)
        #inline_content_new._setId('inline_content')
	#self._setObject('inline_content', inline_content_new, set_owner=0)
        #self.manage_delObjects('inline_content_tmp')

    def cmsPageContentChangeDiscard(self):
      if 'inline_content_tmp' in self.objectIds():
        self.manage_delObjects('inline_content_tmp')

    def cmsPagePublishStatus(self):
     try:
      publish_status = self._is_published
     except:
      publish_status = 1
     return publish_status

    def cmsChangePagePublishStatus(self):
       if self._is_published == 1:
          self.manage_changeProperties(_is_published=0)
       else:
          self.manage_changeProperties(_is_published=1)
       return self._is_published
          
    security.declareProtected( CMFCorePermissions.ModifyPortalContent, 'cmsadmin')
    def cmsadmin(self, REQUEST=Request(url='')):
     """
     main content administration view
     """
     return_message = ''
     form_data = []
     request = REQUEST

     if hasattr(request, 'keys'):
      form_data = request.keys()
     else: 
      form_data = []

     if 'cms_view' in form_data:
       cms_view = request.cms_view
       if len(cms_view) == 0:
         cms_view = 'page_edit'
     else:
       cms_view = 'page_edit'
     # throws an error when request is None, {}, or otherwise empty

     if hasattr(request, 'keys'):
       request.set('cms_view', cms_view)

     if 'edit_object_cms_type' in form_data:
       edit_object_cms_type = request.edit_object_cms_type
     else:
       edit_object_cms_type = None

     if 'cms_edit_object_id' in form_data:
       object_id = request.cms_edit_object_id
     else:
       object_id = None

     if 'cms_action' in form_data:
       cms_action = request.cms_action
     else:
       cms_action = None

     if 'cms_action_workflow' in form_data:
       cms_action_workflow = request.cms_action_workflow
     else:
       cms_action_workflow = None

     if cms_action_workflow == 'save_changes':
	return_message = self.cmsSaveChanges(REQUEST=REQUEST)
	cms_action = None
	request.set('cms_action', cms_action)
	cms_action_workflow = None
	request.set('cms_action_workflow', cms_action_workflow)
	cms_view = 'page_edit'
        request.set('cms_view', cms_view)

     if cms_action:
       if cms_action == 'add':
	 # note: this causes the bug where page adds cause the appearance of uncommited changes
	 #	 this should be handled in the CMSFolder/cmsSaveChanges method
	 #container_folder =  self.cmsPageVersionCheck()
         cms_view = 'object_edit'
         request.set('cms_view', cms_view)
       if cms_action == 'edit':
         cms_view = 'object_edit'
	 # note: this causes the bug where page adds cause the appearance of uncommited changes
	 #	 this should be handled in the CMSFolder/cmsSaveChanges method
	 container_folder =  self.cmsPageVersionCheck()
	 if object_id:
	   edit_object_cms_type = self.inline_content_tmp.cms_get_edit_object_cms_type(object_id)
         request.set('edit_object_cms_type', edit_object_cms_type)
         request.set('cms_view', cms_view)
       if cms_action == 'publish':
         cms_publish_status=self.cmsChangePagePublishStatus()
	 if cms_publish_status:
	   publish_status = 'published'
	 else:
	   publish_status = 'unpublished'
	 return_message = return_message + "<script type='text/javascript' language='JavaScript'> alert('Page is now " + publish_status + ".'); </script>"
         request.set(cms_action, None)
         request.set(cms_publish_status, cms_publish_status)
       if cms_action == 'unpublish':
         request.set(cms_action, None)
       if cms_action == 'commit':
         self.cmsPageContentChangeCommit()
         request.set(cms_action, None)
       if cms_action == 'discard':
         self.cmsPageContentChangeDiscard()
         request.set(cms_action, None)
	 return_message = return_message + "<script type='text/javascript' language='JavaScript'> alert('Changes Discarded.'); </script>"
       if cms_action == 'moveup':
         if self.cms_get_edit_object_cms_type(object_id) == 'page':
	   container_folder = self
	 elif self.cms_get_edit_object_cms_type(object_id) == 'file':
	   container_folder =  self.CMS_files
	 else:
	   container_folder =  self.cmsPageVersionCheck()
         container_folder.move_objects_up(object_id)
       if cms_action == 'movedown':
         if self.cms_get_edit_object_cms_type(object_id) == 'page':
	   container_folder = self
	 elif self.cms_get_edit_object_cms_type(object_id) == 'file':
	   container_folder =  self.CMS_files
	 else:
	   container_folder =  self.cmsPageVersionCheck()
         container_folder.move_objects_down(object_id)
       if cms_action == 'deleteobject':
         if self.cms_get_edit_object_cms_type(object_id) == 'page':
	   container_folder = self
	 elif self.cms_get_edit_object_cms_type(object_id) == 'file':
	   container_folder =  self.CMS_files
	 else:
	   container_folder =  self.cmsPageVersionCheck()
         container_folder.manage_delObjects([object_id])
	 #container_folder.cms_delete_object(object_id)
       if cms_action == 'deletepage':
         request.PARENTS[1].manage_delObjects([object_id])
         return_message = "<script type='text/javascript' language='JavaScript'> alert('Page & All Sub-pages have been deleted.'); window.location='" + request.URL2 + "/cmsadmin'; </script>"
	 return request.PARENTS[1].cmsadmin() 

     #return_message = "<script type='text/javascript' language='JavaScript'> alert(' " + cms_action + " , " + cms_view + " , " + cms_action_workflow + " '); </script>"

     if cms_view == 'page_edit':
       return return_message + self.cms_page_edit()

     if cms_view == 'object_edit':
       return self.cms_object_edit()

     if cms_view == 'page_preview':
       return self.cms_page_preview()

     if cms_view == 'page_properties':
       return self.cms_page_properties()

     return self.cms_page_edit()


    security.declareProtected(CMFCorePermissions.ManagePortal, 'cmsMigrateNonCMS')
    def cmsMigrateNonCMS(self, source_folder, destination_cmspage, REQUEST=None):

      for stxdoc in source_folder.objectValues(['Structured Document']):
        id = stxdoc.getId()
        destination_cmspage.inline_content.invokeFactory(id=id, title=stxdoc.title, type_name='Document')
        new_document = getattr(destination_cmspage.inline_content, id)
        new_document.edit( text_format='structured-text' , text=stxdoc.document_src() ) 

      for dtmldoc in source_folder.objectValues(['DTML Method','DTML Document']):
        id = dtmldoc.getId()
        destination_cmspage.inline_content.invokeFactory(id=id, title=dtmldoc.title, type_name='Document')
        new_document = getattr(destination_cmspage.inline_content, id)
        new_document.edit( text_format='html' , text=dtmldoc.document_src() ) 

      for image in source_folder.objectValues(['Image']):
        id = image.getId()
        destination_cmspage.inline_content.invokeFactory(id=id, title=image.title, type_name='Image')
        new_image = getattr(destination_cmspage.inline_content, id)
        new_image.update_data(image.data, image.content_type, image.get_size())

      for file in source_folder.objectValues(['File']):
        id = file.getId()
        destination_cmspage.CMS_files.invokeFactory(id=id, title=file.title, type_name='File')
        new_file = getattr(destination_cmspage.CMS_files, id)
        new_file.update_data(file.data, file.content_type, file.get_size())

      for folder in source_folder.objectValues(['Folder']):
        destination_cmspage.invokeFactory(id=folder.id, title=folder.title, type_name='CMS Page')
        new_cmspage = getattr(destination_cmspage, folder.id)
        new_cmspage.cmsMigrateNonCMS( source_folder=folder, destination_cmspage=new_cmspage)
     
    def browserType(self, ident):
       if ( (string.find(ident, 'MSIE') == -1)   and (string.find(ident, 'Mozilla') >= 0 ) ):
          if ( (string.find(ident, 'Gecko') > -1 )) :
              ident = 'moz1'
          else:
              ident = 'nscp4'
       elif (string.find(ident, 'MSIE') >= 0):
          ident = 'msie'
          thestr=int(HUSER[ind+4] + HUSER[ind+5])
          if(thestr >= 3):
             ver = 3
          elif(thestr >= 4):
             ver = 4
          elif(thestr >= 5):
             ver = 5
          elif(thestr >= 6):
             ver = 6
          if(ver): ident = ident+str(ver)
       else:
          ident = 'unknown'
       return ident

# note: this is a hack to allow __init__.py/context.registerClass to work so copy/paste support works
addCMSPageForm = DTMLFile('dtml/manage_main', globals())
meta_type			= 'CMS Page'

security.declareProtected(CMFCorePermissions.AddPortalFolders, 'addCMSPage')
def addCMSPage( self, id, title='', description='', REQUEST=None ):
    """ adds a CMS Page """
    sf = CMSPage( id, title=title)
    sf.description=description
    sf._is_published=0
    self._setObject( id, sf )
    obj = getattr(self, id)
    obj.cmsAddContentFolders()
    if REQUEST is not None:
       REQUEST['RESPONSE'].redirect( sf.absolute_url() + '/cmsadmin' )

InitializeClass(CMSPage)
